@@ -73,7 +73,7 @@ sub __drop {
     if ( $type ne 'view' ) {
         $type = 'table';
     }
-    $sql->{table} = $ax->quote_table( $sf->{d}{tables_info}{$table} );
+    $sql->{table} = $ax->qq_table( $sf->{d}{tables_info}{$table} );
     my $stmt_type = 'Drop_' . ucfirst( $type );
     $sf->{d}{stmt_types} = [ $stmt_type ];
     my $info = $ax->get_sql_info( $sql );
