@@ -3,15 +3,15 @@ App::DBBrowser::From::Cte;
 
 use warnings;
 use strict;
-use 5.016;
+use 5.014;
 
-use List::MoreUtils qw( any );
+use List::MoreUtils qw( any firstidx );
 
 use Term::Choose         qw();
 use Term::Form::ReadLine qw();
 
 use App::DBBrowser::Auxil;
-use App::DBBrowser::From::Subquery;
+use App::DBBrowser::Subquery;
 
 sub new {
     my ( $class, $info, $options, $d ) = @_;
@@ -28,14 +28,15 @@ sub cte_as_main_table {
     my ( $sf ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     $sf->{d}{stmt_types} = [ 'Select' ];
-    my $sql = {};
+    my $sql = { table => '()' };
     $ax->reset_sql( $sql );
     $ax->print_sql_info( $ax->get_sql_info( $sql ) ); ##
     my $cte = $sf->cte( $sql );
     if ( ! defined $cte ) {
         return;
     }
-    return $cte;
+    my $qt_table = $cte;
+    return $qt_table;
 }
 
 
@@ -43,23 +44,24 @@ sub cte {
     my ( $sf, $sql ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my ( $new_cte, $remove_ctes ) = ( '  New CTE', '  Remove CTEs' );
+    my ( $new_cte, $reset_unused ) = ( '  New CTE', '  Reset' );
+    my @pre = ( undef );
     my $old_idx = 0;
 
     CHOOSE_CTE: while ( 1 ) {
-        my @avail_ctes = map { '- ' . $_->{name} } @{$sf->{d}{cte_history}} ;
-        my @pre;
+        my $ctes = [ @{$sf->{d}{cte_history}} ];
+        my @avail_ctes = map { '- ' . $_->{name} } @$ctes;
+        my $menu;
         if ( $sf->{d}{nested_subqueries} ) {
-            @pre = ( undef );
+            $menu = [ @pre, @avail_ctes ];
             if ( ! @avail_ctes ) {
                 $ax->print_error_message( 'No ctes available.' );
                 return;
             }
         }
         else {
-            @pre = ( undef, $new_cte, $remove_ctes );
+            $menu = [ @pre, @avail_ctes, $new_cte, $reset_unused ];
         }
-        my $menu = [ @pre, @avail_ctes ];
         my $info = $ax->get_sql_info( $sql );
         # Choose
         my $idx = $tc->choose(
@@ -78,14 +80,23 @@ sub cte {
             $old_idx = $idx;
         }
         if ( $menu->[$idx] eq $new_cte ) {
-            $sf->__prepare_and_add_cte( $sql );
+            $sf->__prepare_and_add_cte( $sql, $ctes );
+            $old_idx++;
         }
-        elsif ( $menu->[$idx] eq $remove_ctes ) {
-            $sf->__remove_ctes( $sql );
+        elsif ( $menu->[$idx] eq $reset_unused ) {
+            my $stmt_type = $sf->{d}{stmt_types}[0];
+            my $bu_cte_history = [ @{$sf->{d}{cte_history}} ];
+            $sf->{d}{cte_history} = [];
+            my $stmt = $ax->get_stmt( $sql, $stmt_type, 'prepare' );
+            $sf->{d}{cte_history} = $bu_cte_history;
+            if ( $stmt_type eq 'Join' ) {
+                $stmt = "SELECT * FROM " . $stmt;
+            }
+            $sf->__reset_unused_ctes( $sql, $ctes, $stmt );
         }
         else {
             $idx -= @pre;
-            my $cte_name = $sf->{d}{cte_history}[$idx]{name};
+            my $cte_name = $ctes->[$idx]{name};
             return $cte_name;
         }
     }
@@ -93,12 +104,11 @@ sub cte {
 
 
 sub __prepare_and_add_cte {
-    my ( $sf, $sql ) = @_;
+    my ( $sf, $sql, $ctes ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $sq = App::DBBrowser::From::Subquery->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $sq = App::DBBrowser::Subquery->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $tr = Term::Form::ReadLine->new( $sf->{i}{tr_default} );
-    my $ctes = [ @{$sf->{d}{cte_history}} ];
 
     CHOOSE_QUERY: while ( 1 ) {
         my $selected_stmt = $sq->__choose_query( $sql, 'cte' );
@@ -108,23 +118,23 @@ sub __prepare_and_add_cte {
         my $count_query_loop;
 
         QUERY: while ( 1 ) {
-            my $info = $ax->get_sql_info( $sql ); ##
+            my $info = $ax->get_sql_info( $sql );
             # Readline
             my $query = $tr->readline(
-                'Stmt: ',
+                'Cte stmt: ',
                 { default => $selected_stmt, show_context => 1, info => $info }
             );
             $ax->print_sql_info( $info );
             if ( ! length $query ) {
                 next CHOOSE_QUERY;
             }
-            $info .= "\n". 'Stmt: ' . $query;
+            $info .= "\n". 'Cte stmt: ' . $query;
             my $default_name;
 
             NAME: while ( 1 ) {
                 # Readline
                 my $full_cte_name = $tr->readline(
-                    'Name: ',
+                    'Cte name: ',
                     { default => $default_name, show_context => 1, info => $info, history => [ 'cte1' .. 'cte9' ] }
                 );
                 $ax->print_sql_info( $info );
@@ -140,10 +150,10 @@ sub __prepare_and_add_cte {
                     name => $cte_name,
                 };
                 $default_name = length $default_name ? '' : $full_cte_name;
-                my @table_keys = keys %{$sf->{d}{tables_info}};
+                my @table_names = keys %{$sf->{d}{tables_info}};
                 my @cte_names = map { $_->{name} } @$ctes;
-                if ( any { $_ eq $cte_name } @table_keys, @cte_names ) {
-                    my $type = ( grep { $_ eq $cte_name } @table_keys ) ? 'table' : 'CTE';
+                if ( any { $_ eq $cte_name } @table_names, @cte_names ) {
+                    my $type = ( firstidx { $_ eq $cte_name } @table_names ) > -1 ? 'table' : 'cte';
                     my $prompt = "A $type '$cte_name' already exists.";
                     # Choose
                     $tc->choose(
@@ -168,51 +178,61 @@ sub __prepare_and_add_cte {
 }
 
 
-sub __remove_ctes {
-    my ( $sf, $sql ) = @_;
-    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my $confirm = $sf->{i}{_confirm};
-    my $all = '  All CTEs';
-    my $ctes = [ @{$sf->{d}{cte_history}} ];
-    my @bu;
+sub __reset_unused_ctes {
+    my ( $sf, $sql, $ctes, $stmt ) = @_;
+    my $all_tables_used_in_stmts = $sf->__table_names_in_query( $stmt );
 
-    REMOVE_CTE: while ( 1 ) {
-        $sf->{d}{cte_history} = [ @$ctes ];
-        my @pre = ( undef, $confirm );
-        my $menu = [ @pre, map( '- ' . $_->{name}, @$ctes ) ];
-        push @$menu, $all;
-        my $info = $ax->get_sql_info( $sql );
-        # Choose
-        my $idx = $tc->choose(
-            $menu,
-            { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Remove CTE:', index => 1, undef => $sf->{i}{_back} }
-        );
-        $ax->print_sql_info( $info );
-        if ( ! defined $idx || $idx == 0 ) {
-            if ( @bu ) {
-                $ctes = pop @bu;
-                next REMOVE_CTE;
-            }
-            return;
-        }
-        push @bu, [ @$ctes ];
-        if ( $menu->[$idx] eq $confirm ) {
-            $sf->{d}{cte_history} = [ @$ctes ];
-            return 1;
-        }
-        elsif ( $menu->[$idx] eq $all ) {
-            $ctes = [];
-        }
-        else {
-            $idx -= @pre;
-            splice( @$ctes, $idx, 1 );
+    CTE: for my $cte ( reverse @$ctes ) {
+        if ( any { $_ eq $cte->{name} } @$all_tables_used_in_stmts ) {
+            my $tables_in_cte = $sf->__table_names_in_query( $cte->{query} );
+            push @$all_tables_used_in_stmts, @$tables_in_cte;
+            $cte->{keep} = 1;
         }
     }
+    $ctes = [ grep { delete $_->{keep} } @$ctes ];
+    $sf->{d}{cte_history} = [ @$ctes ];
+}
+
+
+sub __table_names_in_query {
+    my ( $sf, $stmt ) = @_;
+    my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $rx_quoted_literal = $ax->regex_quoted_literal();
+    my $rx_quoted_identifier = $ax->regex_quoted_identifier();
+    my $rx_sql_elem = qr/ (?: $rx_quoted_identifier | [^',\s]+ ) /x;
+    $rx_sql_elem = qr/ $rx_sql_elem (?: \. $rx_sql_elem ){0,2} /x;
+    my $rx_split_stmt = qr/ ( $rx_quoted_literal | $rx_sql_elem | \s*,\s* ) \s+ /x;
+    my $tables = [];
+    my $implicit_join;
+    my @token = grep { length } split $rx_split_stmt, $stmt;
+
+    while ( @token ) {
+        my $t = shift @token;
+        if( $t =~ /^JOIN\z/i ) {
+            if ( @token ) {
+                push @$tables, shift @token;
+            }
+        }
+        elsif ( $t =~ /^FROM\z/i || $implicit_join ) {
+            if ( @token ) {
+                push @$tables, shift @token;
+            }
+            if ( defined $token[1] && $token[1] eq ',' ) {
+                shift @token;
+            }
+            elsif ( defined $token[2] && $token[2] eq ',' && $token[0] =~ /^AS\z/i ) {
+                shift @token;
+                shift @token;
+            }
+            $implicit_join = defined $token[0] && $token[0] eq ',' ? 1 : 0;
+        }
+    }
+    return $tables;
 }
 
 
 
 1;
 
+
 __END__
