@@ -3,7 +3,7 @@ App::DBBrowser::From::Join;
 
 use warnings;
 use strict;
-use 5.016;
+use 5.014;
 
 use List::MoreUtils qw( any uniq );
 
@@ -13,10 +13,14 @@ use Term::TablePrint     qw();
 
 use App::DBBrowser::Auxil;
 use App::DBBrowser::From::Cte;
-use App::DBBrowser::From::Subquery;
-use App::DBBrowser::Table::Substatement::Condition;
+use App::DBBrowser::Subquery;
+use App::DBBrowser::Table::Substatements;
 
 
+my $join_info      = '  Info';
+my $subquery_table = '  Subquery';
+my $cte_table      = '  Cte';
+
 sub new {
     my ( $class, $info, $options, $d ) = @_;
     my $sf = {
@@ -24,40 +28,30 @@ sub new {
         o => $options,
         d => $d
     };
-    bless $sf, $class;
-}
-
-
-sub __available_joins {
-    my ( $sf ) = @_;
-    if ( $sf->{i}{driver} eq 'SQLite' ) {
-        return [ 'INNER JOIN', 'LEFT JOIN', 'CROSS JOIN' ];
+    if ( $info->{driver} eq 'SQLite' ) {
+        $sf->{join_types} = [ 'INNER JOIN', 'LEFT JOIN', 'CROSS JOIN' ];
     }
-    elsif ( $sf->{i}{driver} =~ /^(?:mysql|MariaDB)\z/ ) {
-        return [ 'INNER JOIN', 'LEFT JOIN', 'RIGHT JOIN', 'CROSS JOIN' ];
+    elsif ( $info->{driver} =~ /^(?:mysql|MariaDB)\z/ ) {
+        $sf->{join_types} = [ 'INNER JOIN', 'LEFT JOIN', 'RIGHT JOIN', 'CROSS JOIN' ];
     }
     else {
-        return [ 'INNER JOIN', 'LEFT JOIN', 'RIGHT JOIN', 'FULL JOIN', 'CROSS JOIN' ];
+        $sf->{join_types} = [ 'INNER JOIN', 'LEFT JOIN', 'RIGHT JOIN', 'FULL JOIN', 'CROSS JOIN' ];
     }
+    bless $sf, $class;
 }
 
 
-my $join_info     = '  Info';
-my $from_subquery = '  Subquery';
-my $from_cte      = '  Cte';
-
-
 sub join_tables {
     my ( $sf ) = @_;
     $sf->{d}{stmt_types} = [ 'Join' ];
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $table_keys;
+    my $tables;
     if ( $sf->{o}{G}{metadata} ) {
-        $table_keys = [ @{$sf->{d}{user_table_keys}}, @{$sf->{d}{sys_table_keys}} ];
+        $tables = [ @{$sf->{d}{user_table_keys}}, @{$sf->{d}{sys_table_keys}} ];
     }
     else {
-        $table_keys = [ @{$sf->{d}{user_table_keys}} ];
+        $tables = [ @{$sf->{d}{user_table_keys}} ];
     }
     my ( $sql, $data );
     my $old_idx_master = 0;
@@ -70,16 +64,16 @@ sub join_tables {
             aliases => [],
             default_alias => 'a',
         };
-        my @choices = map { "- $_" } @$table_keys;
-        push @choices, $from_subquery if $sf->{o}{enable}{j_derived};
-        push @choices, $from_cte      if $sf->{o}{enable}{j_cte};
+        my @choices = map { "- $_" } @$tables;
+        push @choices, $subquery_table if $sf->{o}{enable}{j_derived};
+        push @choices, $cte_table      if $sf->{o}{enable}{j_cte};
         push @choices, $join_info;
         my @pre = ( undef );
         my $menu = [ @pre, @choices ];
         my $info = $ax->get_sql_info( $sql );
         my $idx_master = $tc->choose(
             $menu,
-            { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'The Main Table:', index => 1, default => $old_idx_master }
+            { %{$sf->{i}{lyt_v}}, info => $info, prompt => 'Choose MAIN table:', index => 1, default => $old_idx_master }
         );
         $ax->print_sql_info( $info );
         if ( ! defined $idx_master || ! defined $menu->[$idx_master] ) {
@@ -92,38 +86,40 @@ sub join_tables {
             }
             $old_idx_master = $idx_master;
         }
-        my $master_key = $menu->[$idx_master];
-        if ( $master_key eq $join_info ) {
+        my $master = $menu->[$idx_master];
+        if ( $master eq $join_info ) {
             $sf->__get_join_info();
             $sf->__print_join_info();
             next MASTER;
         }
-        my $master;
-        if ( $master_key eq $from_subquery ) {
-            my $sq = App::DBBrowser::From::Subquery->new( $sf->{i}, $sf->{o}, $sf->{d} );
+        my $qt_master;
+        if ( $master eq $subquery_table ) {
+            my $sq = App::DBBrowser::Subquery->new( $sf->{i}, $sf->{o}, $sf->{d} );
             $master = $sq->subquery( $sql );
             if ( ! defined $master ) {
                 next MASTER;
             }
+            $qt_master = $master;
         }
-        elsif ( $master_key eq $from_cte ) {
+        elsif ( $master eq $cte_table ) {
             my $sq = App::DBBrowser::From::Cte->new( $sf->{i}, $sf->{o}, $sf->{d} );
             $master = $sq->cte( $sql );
             if ( ! defined $master ) {
                 next MASTER;
             }
+            $qt_master = $master;
         }
         else {
-            $master_key =~ s/^-\s//;
-            $master = $ax->qq_table( $sf->{d}{tables_info}{$master_key} );
+            $master =~ s/^-\s//;
+            $qt_master = $ax->qq_table( $sf->{d}{tables_info}{$master} );
         }
         push @{$data->{used_tables}}, $master;
         $data->{default_alias} = 'a';
         # Alias
-        my $master_alias = $ax->table_alias( $sql, 'tables_in_join', $master, $data->{default_alias} );
-        push @{$data->{aliases}}, [ $master, $master_alias ];
-        push @{$sql->{join_data}}, { table => $master . " " . $master_alias };
-        ( $data->{col_names}{$master}, undef ) = $ax->column_names_and_types( $master . " " . $master_alias );
+        my $qt_master_alias = $ax->table_alias( $sql, 'tables_in_join', $qt_master, $data->{default_alias} );
+        push @{$data->{aliases}}, [ $master, $ax->unquote_identifier( $qt_master_alias ) ];
+        push @{$sql->{join_data}}, { table => $qt_master . " " . $qt_master_alias };
+        ( $data->{col_names}{$master}, undef ) = $ax->column_names_and_types( $qt_master . " " . $qt_master_alias );
         if ( ! defined $data->{col_names}{$master} ) {
             next MASTER;
         }
@@ -131,10 +127,9 @@ sub join_tables {
         my $old_idx_type = 0;
 
         JOIN_TYPE: while ( 1 ) {
-            my $enough_tables = $sf->{i}{_confirm};
+            my $enough_tables = '  Enough TABLES';
             my @pre = ( undef, $enough_tables );
-            my $avail_joins = $sf->__available_joins();
-            my $menu = [ @pre, map { "- $_" } map { s/\b(.)/uc $1/ger } map { lc } @$avail_joins ];
+            my $menu = [ @pre, map( "- $_", @{$sf->{join_types}} ) ];
             my $info = $ax->get_sql_info( $sql );
             # Choose
             my $idx_type = $tc->choose(
@@ -163,11 +158,11 @@ sub join_tables {
                 }
                 last JOIN_TYPE;
             }
-            my $join_type = uc $menu->[$idx_type];
+            my $join_type = $menu->[$idx_type];
             $join_type =~ s/^-\s//;
             push @bu, $ax->clone_data( $data );
             push @{$sql->{join_data}}, { join_type => $join_type };
-            my $ok = $sf->__add_slave_with_join_condition( $sql, $data, $table_keys );
+            my $ok = $sf->__add_slave_with_join_condition( $sql, $data, $tables );
             if ( ! $ok ) {
                 $data = pop @bu;
                 pop @{$sql->{join_data}}
@@ -193,39 +188,37 @@ sub join_tables {
             #}                                ##
         }
     }
-    my $columns = [];
-    my $aliases = {};
+    my $qt_columns = [];
+    my $qt_aliases = {};
     for my $table ( @{$data->{used_tables}} ) {
         for my $table_alias ( @{$aliases_hash->{$table}} ) {
             for my $col ( @{$data->{col_names}{$table}} ) {
-                my $qq_col = $ax->qualified_identifier( $table_alias, $col );
-                if ( any { $_ eq $qq_col } @$columns ) {
+                my $col_qt = $ax->qualified_identifier( $ax->quote_alias( $table_alias ), $ax->quote_column( $col ) );
+                if ( any { $_ eq $col_qt } @$qt_columns ) {
                     next;
                 }
                 if ( $sf->{o}{alias}{join_columns} && $col_names{lc $col} > 1 ) {
-                    my $unquoted_table_alias = $ax->unquote_identifier( $table_alias );
-                    my $unquoted_col = $ax->unquote_identifier( $col );
-                    my $alias = $ax->alias( $sql, 'join_columns', $qq_col, $unquoted_table_alias . '_' . $unquoted_col );
-                    if ( length $alias ) {
-                        $aliases->{$qq_col} = $alias;
+                    my $qt_alias = $ax->alias( $sql, 'join_columns', $col_qt, $table_alias . '_' . $col );
+                    if ( length $qt_alias ) {
+                        $qt_aliases->{$col_qt} = $qt_alias;
                     }
                 }
-                push @$columns, $qq_col;
+                push @$qt_columns, $col_qt;
             }
         }
     }
-    my $table = $ax->get_stmt( $sql, 'Join', 'prepare' );
-    return $table, $columns, $aliases;
+    my $qt_table = $ax->get_stmt( $sql, 'Join', 'prepare' );
+    return $qt_table, $qt_columns, $qt_aliases;
 }
 
 
 sub __add_slave_with_join_condition {
-    my ( $sf, $sql, $data, $table_keys ) = @_;
+    my ( $sf, $sql, $data, $tables ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $tc = Term::Choose->new( $sf->{i}{tc_default} );
-    my @choices = map { '- '. $_ } @$table_keys;
-    push @choices, $from_subquery if $sf->{o}{enable}{j_derived};
-    push @choices, $from_cte     if $sf->{o}{enable}{j_cte};
+    my @choices = map { '- '. $_ } @$tables;
+    push @choices, $subquery_table if $sf->{o}{enable}{j_derived};
+    push @choices, $cte_table     if $sf->{o}{enable}{j_cte};
     push @choices, $join_info;
     my $old_idx_slave = 0;
 
@@ -255,41 +248,43 @@ sub __add_slave_with_join_condition {
             $sf->__print_join_info();
             next SLAVE;
         }
-        my $slave_key = $menu->[$idx_slave];
+        my $slave = $menu->[$idx_slave];
         my $bu_data = $ax->clone_data( $data );
-        my $slave;
-        if ( $slave_key eq $from_subquery ) {
-            my $sq = App::DBBrowser::From::Subquery->new( $sf->{i}, $sf->{o}, $sf->{d} );
+        my $qt_slave;
+        if ( $slave eq $subquery_table ) {
+            my $sq = App::DBBrowser::Subquery->new( $sf->{i}, $sf->{o}, $sf->{d} );
             $slave = $sq->subquery( $sql );
             if ( ! defined $slave ) {
                 next SLAVE;
             }
+            $qt_slave = $slave;
         }
-        elsif ( $slave_key eq $from_cte ) {
+        elsif ( $slave eq $cte_table ) {
             my $sq = App::DBBrowser::From::Cte->new( $sf->{i}, $sf->{o}, $sf->{d} );
             $slave = $sq->cte( $sql );
             if ( ! defined $slave ) {
                 next SLAVE;
             }
+            $qt_slave = $slave;
         }
         else {
-            $slave_key =~ s/^-\s//;
-            $slave = $ax->qq_table( $sf->{d}{tables_info}{$slave_key} );
+            $slave =~ s/^-\s//;
+            $qt_slave = $ax->qq_table( $sf->{d}{tables_info}{$slave} );
         }
-        push @{$data->{used_tables}}, $slave; # $slave
+        push @{$data->{used_tables}}, $slave;
         # Alias
-        my $slave_alias = $ax->table_alias( $sql, 'tables_in_join', $slave, ++$data->{default_alias} );
-        $sql->{join_data}[-1]{table} = $slave . " " . $slave_alias;
-        push @{$data->{aliases}}, [ $slave, $slave_alias ];
-        ( $data->{col_names}{$slave}, undef ) = $ax->column_names_and_types( $slave . " " . $slave_alias );
+        my $qt_slave_alias = $ax->table_alias( $sql, 'tables_in_join', $qt_slave, ++$data->{default_alias} );
+        $sql->{join_data}[-1]{table} = $qt_slave . " " . $qt_slave_alias;
+        push @{$data->{aliases}}, [ $slave, $ax->unquote_identifier( $qt_slave_alias ) ];
+        ( $data->{col_names}{$slave}, undef ) = $ax->column_names_and_types( $qt_slave . " " . $qt_slave_alias );
         if ( ! defined $data->{col_names}{$slave} ) {
-            $sf->__reset_to_backed_up_join_data( $sql, $data, $bu_data );
+            $sf->__reset_to_backuped_join_data( $sql, $data, $bu_data );
             next SLAVE;
         }
         if ( $sql->{join_data}[-1]{join_type} ne 'CROSS JOIN' ) {
             my $ok = $sf->__add_join_condition( $sql, $data );
             if ( ! $ok ) {
-                $sf->__reset_to_backed_up_join_data( $sql, $data, $bu_data );
+                $sf->__reset_to_backuped_join_data( $sql, $data, $bu_data );
                 next SLAVE;
             }
         }
@@ -299,7 +294,7 @@ sub __add_slave_with_join_condition {
 }
 
 
-sub __reset_to_backed_up_join_data {
+sub __reset_to_backuped_join_data {
      my ( $sf, $sql, $data, $bu_data ) = @_;
     $sql->{join_data}[-1] = { join_type => $sql->{join_data}[-1]{join_type} };
     delete @{$data}{ keys %$data };
@@ -313,7 +308,7 @@ sub __reset_to_backed_up_join_data {
 sub __add_join_condition {
     my ( $sf, $sql, $data ) = @_;
     my $ax = App::DBBrowser::Auxil->new( $sf->{i}, $sf->{o}, $sf->{d} );
-    my $sc = App::DBBrowser::Table::Substatement::Condition->new( $sf->{i}, $sf->{o}, $sf->{d} );
+    my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my $aliases_hash = {};
     for my $ref ( @{$data->{aliases}} ) {
         push @{$aliases_hash->{$ref->[0]}}, $ref->[1];
@@ -322,12 +317,15 @@ sub __add_join_condition {
     for my $used_table ( uniq @{$data->{used_tables}} ) {           # self join: a table name is used more than once
         for my $table_alias ( @{$aliases_hash->{$used_table}} ) {   # self join: a table has more than one alias
             for my $col ( @{$data->{col_names}{$used_table}} ) {
-                push @$cols_join_condition, $ax->qualified_identifier( $table_alias, $col );
+                push @$cols_join_condition, $ax->qualified_identifier(
+                    $ax->quote_alias( $table_alias ),
+                    $ax->quote_column( $col )
+                );
             }
         }
     }
     $sql->{cols_join_condition} = $cols_join_condition;
-    my $ret = $sc->add_condition( $sql, 'on', $cols_join_condition );
+    my $ret = $sb->add_condition( $sql, 'on', $cols_join_condition );
     if ( $ret && length $sql->{on_stmt} ) {
         $sql->{join_data}[-1]{condition} = $sql->{on_stmt};
         $sql->{on_stmt} = '';
@@ -368,35 +366,35 @@ sub __get_join_info {
     my ( $sf ) = @_;
     return if $sf->{d}{pk_info};
     my $td = $sf->{d}{tables_info};
-    my $table_keys;
+    my $tables;
     if ( $sf->{o}{G}{metadata} ) {
-        $table_keys = [ @{$sf->{d}{user_table_keys}}, @{$sf->{d}{sys_table_keys}} ];
+        $tables = [ @{$sf->{d}{user_table_keys}}, @{$sf->{d}{sys_table_keys}} ];
     }
     else {
-        $table_keys = [ @{$sf->{d}{user_table_keys}} ];
+        $tables = [ @{$sf->{d}{user_table_keys}} ];
     }
     my $pk = {};
-    for my $table_key ( @$table_keys ) {
-        my $sth = $sf->{d}{dbh}->primary_key_info( @{$td->{$table_key}}[0..2] );
+    for my $table ( @$tables ) {
+        my $sth = $sf->{d}{dbh}->primary_key_info( @{$td->{$table}}[0..2] );
         next if ! defined $sth;
         while ( my $ref = $sth->fetchrow_hashref() ) {
             next if ! defined $ref;
-            #$pk->{$table_key}{TABLE_SCHEM} =        $ref->{TABLE_SCHEM};
-            $pk->{$table_key}{TABLE_NAME}  =        $ref->{TABLE_NAME};
-            push @{$pk->{$table_key}{COLUMN_NAME}}, $ref->{COLUMN_NAME};
-            #push @{$pk->{$table_key}{KEY_SEQ}},     $ref->{KEY_SEQ} // $ref->{ORDINAL_POSITION};
+            #$pk->{$table}{TABLE_SCHEM} =        $ref->{TABLE_SCHEM};
+            $pk->{$table}{TABLE_NAME}  =        $ref->{TABLE_NAME};
+            push @{$pk->{$table}{COLUMN_NAME}}, $ref->{COLUMN_NAME};
+            #push @{$pk->{$table}{KEY_SEQ}},     $ref->{KEY_SEQ} // $ref->{ORDINAL_POSITION};
         }
     }
     my $fk = {};
-    for my $table_key ( @$table_keys ) {
-        my $sth = $sf->{d}{dbh}->foreign_key_info( @{$td->{$table_key}}[0..2], undef, undef, undef );
+    for my $table ( @$tables ) {
+        my $sth = $sf->{d}{dbh}->foreign_key_info( @{$td->{$table}}[0..2], undef, undef, undef );
         next if ! defined $sth;
         while ( my $ref = $sth->fetchrow_hashref() ) {
             next if ! defined $ref;
-            #$fk->{$table_key}{FKTABLE_SCHEM} =        $ref->{FKTABLE_SCHEM} // $ref->{FK_TABLE_SCHEM};
-            $fk->{$table_key}{FKTABLE_NAME}  =        $ref->{FKTABLE_NAME} // $ref->{FK_TABLE_NAME};
-            push @{$fk->{$table_key}{FKCOLUMN_NAME}}, $ref->{FKCOLUMN_NAME} // $ref->{FK_COLUMN_NAME};
-            #push @{$fk->{$table_key}{KEY_SEQ}},       $ref->{KEY_SEQ} // $ref->{ORDINAL_POSITION};
+            #$fk->{$table}{FKTABLE_SCHEM} =        $ref->{FKTABLE_SCHEM} // $ref->{FK_TABLE_SCHEM};
+            $fk->{$table}{FKTABLE_NAME}  =        $ref->{FKTABLE_NAME} // $ref->{FK_TABLE_NAME};
+            push @{$fk->{$table}{FKCOLUMN_NAME}}, $ref->{FKCOLUMN_NAME} // $ref->{FK_COLUMN_NAME};
+            #push @{$fk->{$table}{KEY_SEQ}},       $ref->{KEY_SEQ} // $ref->{ORDINAL_POSITION};
         }
     }
     $sf->{d}{pk_info} = $pk;
