@@ -33,9 +33,6 @@ sub table_write_access {
     my $sb = App::DBBrowser::Table::Substatements->new( $sf->{i}, $sf->{o}, $sf->{d} );
     my @stmt_types;
     if ( $sf->{d}{table_origin} eq 'ordinary' ) {
-        # ordinary tables: use tablename without alias:
-        my $table_key = $sf->{d}{table_key};
-        $sql->{table} = $ax->quote_table( $sf->{d}{tables_info}{$table_key} );
         push @stmt_types, 'Insert' if $sf->{o}{enable}{insert_into};
         push @stmt_types, 'Update' if $sf->{o}{enable}{update};
         push @stmt_types, 'Delete' if $sf->{o}{enable}{delete};
@@ -72,6 +69,11 @@ sub table_write_access {
         $sf->{d}{stmt_types} = [ $stmt_type ];
         $ax->reset_sql( $sql );
         if ( $stmt_type eq 'Insert' ) {
+            #if ( $sf->{d}{table_origin} eq 'ordinary' ) {
+                # Insert: use tablename without alias:
+                my $table_key = $sf->{d}{table_key};
+                $sql->{table} = $ax->qq_table( $sf->{d}{tables_info}{$table_key} );
+            #}
             my $ok = $sf->__build_insert_stmt( $sql );
             if ( $ok ) {
                 $cs->commit_sql( $sql );
